import React from 'react';
import uuid from 'node-uuid';
require('./style.css');

class NotificationItem {
  constructor(message, type, timeout) {
    this.message = message;
    this.type = type || 'info';
    this.timeout = timeout || 3000;

    this.id = uuid.v4();
  }
}

class NotificationRow extends React.Component {
  constructor(props) {
    super(props);

    this.state = { toFade: false };

    this.handleClose = this.close.bind(this);
  }

  componentDidMount() {
    this.mounted = true;

    const { notification } = this.props;
    const timeout = notification.timeout || 3000;
    const self = this;
    setTimeout(() => {
      self.close();
    }, timeout);
  }

  componentWillUnmount() {
    this.mounted = false;
  }

  close() {
    if (this.mounted) this.setState({ toFade: true });

    const { notification, onFadeout } = this.props;
    setTimeout(() => {
      onFadeout(notification);
    }, 1000);
  }

  render() {
    const { notification } = this.props;
    const { toFade } = this.state;
    return (
      <div className="row">
        <div className={ `alert alert-${notification.type} alert-dismissible fadeNotification ${toFade ? 'fadeout' : ''}` } role="alert" style={ { textAlign: 'center' } }>
          <button onClick={ this.handleClose } type="button" className="close" data-dismiss="alert" aria-label="Close">
            <span aria-hidden="true">&times;</span>
          </button>
          { notification.message }
          <strong> { notification.type.toUpperCase() } </strong>
        </div>
      </div>
    );
  }
}

const NotificationList = ({ list, onFadeout }) => (
  <div className="container">
    {
      list.map((item, index) => <NotificationRow key={ index } notification={ item } onFadeout={ onFadeout } />)
    }
  </div>
);


const { array, object, func, number } = React.PropTypes;
NotificationList.propTypes = {
  list: array,

  onFadeout: func,
};

NotificationRow.propTypes = {
  notification: object,
  timeout: number,

  onFadeout: func,
};

export default (() => {
  const NotificationComponent = {
    isMounted: false,
    Instance: null,
  };

  class InternalComponent extends React.Component {
    constructor(props) {
      super(props);

      this.state = { notifications: [] };

      this.handleFadeout = this.fadeout.bind(this);
    }

    componentDidMount() {
      NotificationComponent.isMounted = true;
      NotificationComponent.Instance = this;
    }

    componentWillUnmount() {
      this.mounted = false;

      NotificationComponent.isMounted = true;
    }

    fadeout(item) {
      const { notifications } = this.state;
      const newNotifications = notifications.filter((i) => i.id !== item.id);
      this.setState({ notifications: newNotifications });
    }

    reset() {
      this.setState({ notifications: [] });
    }

    add(msg, opt) {
      const notification = new NotificationItem(msg, opt.type);
      const { notifications } = this.state;
      notifications.push(notification);

      this.setState({ notifications });
    }

    render() {
      const { notifications } = this.state;

      return <NotificationList list={ notifications } onFadeout={ this.handleFadeout } />;
    }
  }

  const ReactFlasher = {
    Component: InternalComponent,

    clear: () => {
      if (NotificationComponent.isMounted) {
        NotificationComponent.Instance.reset();
      }
    },

    warning: (msg, opts) => {
      if (NotificationComponent.isMounted) {
        NotificationComponent.Instance.add(msg, { opts, type: 'warning' });
      }
    },
    error: (msg, opts) => {
      if (NotificationComponent.isMounted) {
        NotificationComponent.Instance.add(msg, { opts, type: 'error' });
      }
    },
    success: (msg, opts) => {
      if (NotificationComponent.isMounted) {
        NotificationComponent.Instance.add(msg, { opts, type: 'success' });
      }
    },
    info: (msg, opts) => {
      if (NotificationComponent.isMounted) {
        NotificationComponent.Instance.add(msg, { opts, type: 'info' });
      }
    },
  };

  return ReactFlasher;
})();
